@@ -42,12 +42,6 @@
 
    + p = 0  ==>  r = 0, correct!
 */
-mp_size_t
-ecc_dup_jj_itch (const struct ecc_curve *ecc)
-{
-  return ECC_DUP_JJ_ITCH (ecc->size);
-}
-
 void
 ecc_dup_jj (const struct ecc_curve *ecc,
 	    mp_limb_t *r, const mp_limb_t *p,
@@ -67,50 +61,50 @@ ecc_dup_jj (const struct ecc_curve *ecc,
   */
 
 #define delta  scratch
-#define gamma (scratch + ecc->size)
-#define beta  (scratch + 2*ecc->size)
-#define g2    (scratch + 3*ecc->size)
-#define sum   (scratch + 4*ecc->size)
+#define gamma (scratch + ecc->p.size)
+#define beta  (scratch + 2*ecc->p.size)
+#define g2    (scratch + 3*ecc->p.size)
+#define sum   (scratch + 4*ecc->p.size)
 #define alpha  scratch /* Overlap delta */
   
 #define xp p
-#define yp (p + ecc->size)
-#define zp (p + 2*ecc->size)
+#define yp (p + ecc->p.size)
+#define zp (p + 2*ecc->p.size)
   
   /* delta */
-  ecc_modp_sqr (ecc, delta, zp);
+  ecc_mod_sqr (&ecc->p, delta, zp);
 
   /* gamma */
-  ecc_modp_sqr (ecc, gamma, yp);
+  ecc_mod_sqr (&ecc->p, gamma, yp);
 
   /* z'. Can use beta area as scratch. */
-  ecc_modp_add (ecc, r + 2*ecc->size, yp, zp);
-  ecc_modp_sqr (ecc, beta, r + 2*ecc->size);
-  ecc_modp_sub (ecc, beta, beta, gamma);
-  ecc_modp_sub (ecc, r + 2*ecc->size, beta, delta);
+  ecc_mod_add (&ecc->p, r + 2*ecc->p.size, yp, zp);
+  ecc_mod_sqr (&ecc->p, beta, r + 2*ecc->p.size);
+  ecc_mod_sub (&ecc->p, beta, beta, gamma);
+  ecc_mod_sub (&ecc->p, r + 2*ecc->p.size, beta, delta);
   
   /* alpha. Can use beta area as scratch, and overwrite delta. */
-  ecc_modp_add (ecc, sum, xp, delta);
-  ecc_modp_sub (ecc, delta, xp, delta);
-  ecc_modp_mul (ecc, beta, sum, delta);
-  ecc_modp_mul_1 (ecc, alpha, beta, 3);
+  ecc_mod_add (&ecc->p, sum, xp, delta);
+  ecc_mod_sub (&ecc->p, delta, xp, delta);
+  ecc_mod_mul (&ecc->p, beta, sum, delta);
+  ecc_mod_mul_1 (&ecc->p, alpha, beta, 3);
 
   /* beta */
-  ecc_modp_mul (ecc, beta, xp, gamma);
+  ecc_mod_mul (&ecc->p, beta, xp, gamma);
 
   /* Do gamma^2 and 4*beta early, to get them out of the way. We can
      then use the old area at gamma as scratch. */
-  ecc_modp_sqr (ecc, g2, gamma);
-  ecc_modp_mul_1 (ecc, sum, beta, 4);
+  ecc_mod_sqr (&ecc->p, g2, gamma);
+  ecc_mod_mul_1 (&ecc->p, sum, beta, 4);
   
   /* x' */
-  ecc_modp_sqr (ecc, gamma, alpha);   /* Overwrites gamma and beta */
-  ecc_modp_submul_1 (ecc, gamma, sum, 2);
-  mpn_copyi (r, gamma, ecc->size);
+  ecc_mod_sqr (&ecc->p, gamma, alpha);   /* Overwrites gamma and beta */
+  ecc_mod_submul_1 (&ecc->p, gamma, sum, 2);
+  mpn_copyi (r, gamma, ecc->p.size);
 
   /* y' */
-  ecc_modp_sub (ecc, sum, sum, r);
-  ecc_modp_mul (ecc, gamma, sum, alpha);
-  ecc_modp_submul_1 (ecc, gamma, g2, 8);
-  mpn_copyi (r + ecc->size, gamma, ecc->size);
+  ecc_mod_sub (&ecc->p, sum, sum, r);
+  ecc_mod_mul (&ecc->p, gamma, sum, alpha);
+  ecc_mod_submul_1 (&ecc->p, gamma, g2, 8);
+  mpn_copyi (r + ecc->p.size, gamma, ecc->p.size);
 }
