@@ -70,12 +70,13 @@ equal_h (const struct ecc_modulo *p,
 mp_size_t
 _eddsa_verify_itch (const struct ecc_curve *ecc)
 {
+  assert (_eddsa_decompress_itch (ecc) <= ecc->mul_itch);
   return 8*ecc->p.size + ecc->mul_itch;
 }
 
 int
 _eddsa_verify (const struct ecc_curve *ecc,
-	       const struct nettle_hash *H,
+	       const struct ecc_eddsa *eddsa,
 	       const uint8_t *pub,
 	       const mp_limb_t *A,
 	       void *ctx,
@@ -105,16 +106,16 @@ _eddsa_verify (const struct ecc_curve *ecc,
   if (mpn_cmp (sp, ecc->q.m, ecc->q.size) >= 0)
     return 0;
 
-  H->init (ctx);
-  H->update (ctx, nbytes, signature);
-  H->update (ctx, nbytes, pub);
-  H->update (ctx, length, msg);
-  H->digest (ctx, 2*nbytes, hash);
-  _eddsa_hash (&ecc->q, hp, hash);
+  eddsa->dom (ctx);
+  eddsa->update (ctx, nbytes, signature);
+  eddsa->update (ctx, nbytes, pub);
+  eddsa->update (ctx, length, msg);
+  eddsa->digest (ctx, 2*nbytes, hash);
+  _eddsa_hash (&ecc->q, hp, 2*nbytes, hash);
 
   /* Compute h A + R - s G, which should be the neutral point */
   ecc->mul (ecc, P, hp, A, scratch_out);
-  ecc_add_eh (ecc, P, P, R, scratch_out);
+  ecc->add_hh (ecc, P, P, R, scratch_out);
   /* Move out of the way. */
   mpn_copyi (hp, sp, ecc->q.size);
   ecc->mul_g (ecc, S, hp, scratch_out);
