@@ -35,30 +35,59 @@
 #include "testutils.h"
 
 #include "chacha.h"
+#include "chacha-internal.h"
 
 static void
-test_chacha(const struct tstring *key, const struct tstring *nonce,
-	    const struct tstring *expected, unsigned rounds)
+_test_chacha(const struct tstring *key, const struct tstring *nonce,
+	     const struct tstring *expected, unsigned rounds,
+	     const struct tstring *counter)
 {
   struct chacha_ctx ctx;
 
   ASSERT (key->length == CHACHA_KEY_SIZE);
   chacha_set_key (&ctx, key->data);
-  ASSERT (nonce->length == CHACHA_NONCE_SIZE);
 
   if (rounds == 20)
     {
       uint8_t *data = xalloc (expected->length + 2);
-      data++;
       size_t length;
+      data++;
 
       for (length = 1; length <= expected->length; length++)
 	{
 	  data[-1] = 17;
 	  memset (data, 0, length);
 	  data[length] = 17;
-	  chacha_set_nonce(&ctx, nonce->data);
-	  chacha_crypt (&ctx, length, data, data);
+	  if (nonce->length == CHACHA_NONCE_SIZE)
+	    chacha_set_nonce(&ctx, nonce->data);
+	  else if (nonce->length == CHACHA_NONCE96_SIZE)
+	    {
+	      chacha_set_nonce96(&ctx, nonce->data);
+	      /* Use initial counter 1, for
+		 draft-irtf-cfrg-chacha20-poly1305-08 test cases. */
+	      ctx.state[12]++;
+	    }
+	  else
+	    die ("Bad nonce size %u.\n", (unsigned) nonce->length);
+
+	  if (counter)
+	    {
+	      if (counter->length == CHACHA_COUNTER_SIZE)
+		{
+		  ASSERT (nonce->length == CHACHA_NONCE_SIZE);
+		  chacha_set_counter(&ctx, counter->data);
+		}
+	      else if (counter->length == CHACHA_COUNTER32_SIZE)
+		{
+		  ASSERT (nonce->length == CHACHA_NONCE96_SIZE);
+		  chacha_set_counter32(&ctx, counter->data);
+		}
+	    }
+
+	  if (nonce->length == CHACHA_NONCE_SIZE)
+	    chacha_crypt (&ctx, length, data, data);
+	  else
+	    chacha_crypt32 (&ctx, length, data, data);
 
 	  ASSERT (data[-1] == 17);
 	  ASSERT (data[length] == 17);
@@ -84,8 +113,11 @@ test_chacha(const struct tstring *key, const struct tstring *nonce,
 	 numbers of rounds. */
       uint32_t out[_CHACHA_STATE_LENGTH];
       ASSERT (expected->length == CHACHA_BLOCK_SIZE);
+      ASSERT (nonce->length == CHACHA_NONCE_SIZE);
 
       chacha_set_nonce(&ctx, nonce->data);
+      if (counter)
+	    chacha_set_counter(&ctx, counter->data);
       _chacha_core (out, ctx.state, rounds);
 
       if (!MEMEQ(CHACHA_BLOCK_SIZE, out, expected->data))
@@ -105,6 +137,21 @@ test_chacha(const struct tstring *key, const struct tstring *nonce,
     }
 }
 
+static void
+test_chacha(const struct tstring *key, const struct tstring *nonce,
+	    const struct tstring *expected, unsigned rounds)
+{
+  _test_chacha(key, nonce, expected, rounds, NULL);
+}
+
+static void
+test_chacha_with_counter(const struct tstring *key, const struct tstring *nonce,
+			 const struct tstring *expected, unsigned rounds,
+			 const struct tstring *counter)
+{
+  _test_chacha(key, nonce, expected, rounds, counter);
+}
+
 void
 test_main(void)
 {
@@ -622,4 +669,38 @@ test_main(void)
 		   "ae2c4c90225ba9ea 14d518f55929dea0"
 		   "98ca7a6ccfe61227 053c84e49a4a3332"),
 	      20);
+
+  /* From draft-irtf-cfrg-chacha20-poly1305-08, with 96-bit nonce */
+  test_chacha(SHEX("0001020304050607 08090a0b0c0d0e0f"
+		   "1011121314151617 18191a1b1c1d1e1f"),
+	      SHEX("000000090000004a 00000000"),
+	      SHEX("10f1e7e4d13b5915 500fdd1fa32071c4"
+		   "c7d1f4c733c06803 0422aa9ac3d46c4e"
+		   "d2826446079faa09 14c2d705d98b02a2"
+		   "b5129cd1de164eb9 cbd083e8a2503c4e"),
+	      20);
+
+  /* This is identical to the above 96-bit nonce test, but it manually
+     sets the 32-bit counter value */
+  test_chacha_with_counter(SHEX("0001020304050607 08090a0b0c0d0e0f"
+				"1011121314151617 18191a1b1c1d1e1f"),
+			   SHEX("000000090000004a 00000000"),
+			   SHEX("10f1e7e4d13b5915 500fdd1fa32071c4"
+				"c7d1f4c733c06803 0422aa9ac3d46c4e"
+				"d2826446079faa09 14c2d705d98b02a2"
+				"b5129cd1de164eb9 cbd083e8a2503c4e"),
+			   20,
+			   SHEX("01000000"));
+
+  /* This is identical to the above 96-bit nonce test, but it manually
+     sets the 64-bit counter value */
+  test_chacha_with_counter(SHEX("0001020304050607 08090a0b0c0d0e0f"
+				"1011121314151617 18191a1b1c1d1e1f"),
+			   SHEX("0000004a00000000"),
+			   SHEX("10f1e7e4d13b5915 500fdd1fa32071c4"
+				"c7d1f4c733c06803 0422aa9ac3d46c4e"
+				"d2826446079faa09 14c2d705d98b02a2"
+				"b5129cd1de164eb9 cbd083e8a2503c4e"),
+			   20,
+			   SHEX("0100000000000009"));
 }
