@@ -4,6 +4,7 @@
 
 #include "base16.h"
 #include "cbc.h"
+#include "cfb.h"
 #include "ctr.h"
 #include "knuth-lfib.h"
 #include "macros.h"
@@ -62,7 +63,7 @@ tstring_clear(void)
 }
 
 struct tstring *
-tstring_data(size_t length, const char *data)
+tstring_data(size_t length, const uint8_t *data)
 {
   struct tstring *s = tstring_alloc (length);
   memcpy (s->data, data, length);
@@ -244,6 +245,388 @@ test_cipher_cbc(const struct nettle_cipher *cipher,
   free(iv);
 }
 
+void
+test_cipher_cfb(const struct nettle_cipher *cipher,
+		const struct tstring *key,
+		const struct tstring *cleartext,
+		const struct tstring *ciphertext,
+		const struct tstring *iiv)
+{
+  void *ctx = xalloc(cipher->context_size);
+  uint8_t *data, *data2;
+  uint8_t *iv = xalloc(cipher->block_size);
+  size_t length;
+
+  ASSERT (cleartext->length == ciphertext->length);
+  length = cleartext->length;
+
+  ASSERT (key->length == cipher->key_size);
+  ASSERT (iiv->length == cipher->block_size);
+
+  data = xalloc(length);
+  data2 = xalloc(length);
+
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, cleartext->data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB encrypt failed:\nInput:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data2, data);
+
+  if (!MEMEQ(length, data2, cleartext->data))
+    {
+      fprintf(stderr, "CFB decrypt failed:\nInput:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data2);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+  memcpy(data, cleartext->data, length);
+
+  cfb_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB inplace encrypt failed:\nInput:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, cleartext->data))
+    {
+      fprintf(stderr, "CFB inplace decrypt failed:\nInput:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  /* Repeat all tests with incomplete last block */
+  length -= 1;
+
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, cleartext->data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB encrypt failed:\nInput:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data2, data);
+
+  if (!MEMEQ(length, data2, cleartext->data))
+    {
+      fprintf(stderr, "CFB decrypt failed:\nInput:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data2);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+  memcpy(data, cleartext->data, length);
+
+  cfb_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB inplace encrypt failed:\nInput:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, cleartext->data))
+    {
+      fprintf(stderr, "CFB inplace decrypt failed:\nInput:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  free(ctx);
+  free(data);
+  free(data2);
+  free(iv);
+}
+
+void
+test_cipher_cfb8(const struct nettle_cipher *cipher,
+		 const struct tstring *key,
+		 const struct tstring *cleartext,
+		 const struct tstring *ciphertext,
+		 const struct tstring *iiv)
+{
+  void *ctx = xalloc(cipher->context_size);
+  uint8_t *data, *data2;
+  uint8_t *iv = xalloc(cipher->block_size);
+  size_t length;
+  size_t block;
+
+  ASSERT (cleartext->length == ciphertext->length);
+  length = cleartext->length;
+
+  ASSERT (key->length == cipher->key_size);
+  ASSERT (iiv->length == cipher->block_size);
+
+  data = xalloc(length + 1);
+  data2 = xalloc(length + 1);
+
+  for (block = 1; block <= length; block++)
+    {
+      size_t i;
+
+      cipher->set_encrypt_key(ctx, key->data);
+      memcpy(iv, iiv->data, cipher->block_size);
+
+      memset(data, 0x17, length + 1);
+      for (i = 0; i + block <= length; i += block)
+	{
+	  cfb8_encrypt(ctx, cipher->encrypt,
+		       cipher->block_size, iv,
+		       block, data + i, cleartext->data + i);
+	}
+      cfb8_encrypt(ctx, cipher->encrypt,
+		   cipher->block_size, iv,
+		   length - i, data + i, cleartext->data + i);
+
+      if (!MEMEQ(length, data, ciphertext->data))
+	{
+	  fprintf(stderr, "CFB8 encrypt failed, block size %lu:\nInput:",
+		  (unsigned long) block);
+	  tstring_print_hex(cleartext);
+	  fprintf(stderr, "\nOutput: ");
+	  print_hex(length, data);
+	  fprintf(stderr, "\nExpected:");
+	  tstring_print_hex(ciphertext);
+	  fprintf(stderr, "\n");
+	  FAIL();
+	}
+      ASSERT (data[length] == 0x17);
+
+      cipher->set_encrypt_key(ctx, key->data);
+      memcpy(iv, iiv->data, cipher->block_size);
+
+      memset(data2, 0x17, length + 1);
+      for (i = 0; i + block <= length; i += block)
+	{
+	  cfb8_decrypt(ctx, cipher->encrypt,
+		       cipher->block_size, iv,
+		       block, data2 + i, data + i);
+	}
+      cfb8_decrypt(ctx, cipher->encrypt,
+		   cipher->block_size, iv,
+		   length - i, data2 + i, data + i);
+
+      if (!MEMEQ(length, data2, cleartext->data))
+	{
+	  fprintf(stderr, "CFB8 decrypt failed, block size %lu:\nInput:",
+		  (unsigned long) block);
+	  tstring_print_hex(ciphertext);
+	  fprintf(stderr, "\nOutput: ");
+	  print_hex(length, data2);
+	  fprintf(stderr, "\nExpected:");
+	  tstring_print_hex(cleartext);
+	  fprintf(stderr, "\n");
+	  FAIL();
+	}
+      ASSERT (data[length] == 0x17);
+    }
+
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+  memcpy(data, cleartext->data, length);
+
+  cfb8_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB8 inplace encrypt failed:\nInput:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb8_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, cleartext->data))
+    {
+      fprintf(stderr, "CFB8 inplace decrypt failed:\nInput:");
+      tstring_print_hex(ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      tstring_print_hex(cleartext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  /* Repeat all tests with incomplete last block */
+  length -= 1;
+
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb8_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, cleartext->data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB8 encrypt failed:\nInput:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb8_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data2, data);
+
+  if (!MEMEQ(length, data2, cleartext->data))
+    {
+      fprintf(stderr, "CFB8 decrypt failed:\nInput:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data2);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+  memcpy(data, cleartext->data, length);
+
+  cfb8_encrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, ciphertext->data))
+    {
+      fprintf(stderr, "CFB8 inplace encrypt failed:\nInput:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  cipher->set_encrypt_key(ctx, key->data);
+  memcpy(iv, iiv->data, cipher->block_size);
+
+  cfb8_decrypt(ctx, cipher->encrypt,
+	      cipher->block_size, iv,
+	      length, data, data);
+
+  if (!MEMEQ(length, data, cleartext->data))
+    {
+      fprintf(stderr, "CFB8 inplace decrypt failed:\nInput:");
+      print_hex(length, ciphertext->data);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext->data);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  free(ctx);
+  free(data);
+  free(data2);
+  free(iv);
+}
+
 void
 test_cipher_ctr(const struct nettle_cipher *cipher,
 		const struct tstring *key,
@@ -257,6 +640,7 @@ test_cipher_ctr(const struct nettle_cipher *cipher,
   uint8_t *octr = xalloc(cipher->block_size);
   size_t length, nblocks;
   unsigned low;
+  size_t i;
 
   ASSERT (cleartext->length == ciphertext->length);
   length = cleartext->length;
@@ -278,22 +662,29 @@ test_cipher_ctr(const struct nettle_cipher *cipher,
   data = xalloc(length);  
 
   cipher->set_encrypt_key(ctx, key->data);
-  memcpy(ctr, ictr->data, cipher->block_size);
 
-  ctr_crypt(ctx, cipher->encrypt,
-	    cipher->block_size, ctr,
-	    length, data, cleartext->data);
-
-  if (!MEMEQ(length, data, ciphertext->data))
+  for (i = 0; i <= length; i++)
     {
-      fprintf(stderr, "CTR encrypt failed:\nInput:");
-      tstring_print_hex(cleartext);
-      fprintf(stderr, "\nOutput: ");
-      print_hex(length, data);
-      fprintf(stderr, "\nExpected:");
-      tstring_print_hex(ciphertext);
-      fprintf(stderr, "\n");
-      FAIL();
+      memcpy(ctr, ictr->data, cipher->block_size);
+      memset(data, 17, length);
+
+      ctr_crypt(ctx, cipher->encrypt,
+		cipher->block_size, ctr,
+		i, data, cleartext->data);
+
+      if (!MEMEQ(i, data, ciphertext->data)
+	  || (i < length && data[i] != 17))
+	{
+	  fprintf(stderr, "CTR encrypt failed (length %d of %d):\nInput:",
+		  (int) i, (int) length);
+	  tstring_print_hex(cleartext);
+	  fprintf(stderr, "\nOutput: ");
+	  print_hex(length, data);
+	  fprintf(stderr, "\nExpected:");
+	  tstring_print_hex(ciphertext);
+	  fprintf(stderr, "\n");
+	  FAIL();
+	}
     }
 
   ASSERT (MEMEQ (cipher->block_size, ctr, octr));
@@ -475,33 +866,36 @@ test_hash(const struct nettle_hash *hash,
 	  const struct tstring *digest)
 {
   void *ctx = xalloc(hash->context_size);
-  uint8_t *buffer = xalloc(hash->digest_size);
+  uint8_t *buffer = xalloc(digest->length);
   uint8_t *input;
   unsigned offset;
 
-  ASSERT (digest->length == hash->digest_size);
+  /* Here, hash->digest_size zero means arbitrary size. */
+  if (hash->digest_size)
+    ASSERT (digest->length == hash->digest_size);
 
   hash->init(ctx);
   hash->update(ctx, msg->length, msg->data);
-  hash->digest(ctx, hash->digest_size, buffer);
+  hash->digest(ctx, digest->length, buffer);
 
-  if (MEMEQ(hash->digest_size, digest->data, buffer) == 0)
+  if (MEMEQ(digest->length, digest->data, buffer) == 0)
     {
       fprintf(stdout, "\nGot:\n");
-      print_hex(hash->digest_size, buffer);
+      print_hex(digest->length, buffer);
       fprintf(stdout, "\nExpected:\n");
-      print_hex(hash->digest_size, digest->data);
+      print_hex(digest->length, digest->data);
       abort();
     }
 
-  memset(buffer, 0, hash->digest_size);
+  memset(buffer, 0, digest->length);
 
   hash->update(ctx, msg->length, msg->data);
-  hash->digest(ctx, hash->digest_size - 1, buffer);
+  ASSERT(digest->length > 0);
+  hash->digest(ctx, digest->length - 1, buffer);
 
-  ASSERT(MEMEQ(hash->digest_size - 1, digest->data, buffer));
+  ASSERT(MEMEQ(digest->length - 1, digest->data, buffer));
 
-  ASSERT(buffer[hash->digest_size - 1] == 0);
+  ASSERT(buffer[digest->length - 1] == 0);
 
   input = xalloc (msg->length + 16);
   for (offset = 0; offset < 16; offset++)
@@ -509,13 +903,13 @@ test_hash(const struct nettle_hash *hash,
       memset (input, 0, msg->length + 16);
       memcpy (input + offset, msg->data, msg->length);
       hash->update (ctx, msg->length, input + offset);
-      hash->digest (ctx, hash->digest_size, buffer);
-      if (MEMEQ(hash->digest_size, digest->data, buffer) == 0)
+      hash->digest (ctx, digest->length, buffer);
+      if (MEMEQ(digest->length, digest->data, buffer) == 0)
 	{
 	  fprintf(stdout, "hash input address: %p\nGot:\n", input + offset);
-	  print_hex(hash->digest_size, buffer);
+	  print_hex(digest->length, buffer);
 	  fprintf(stdout, "\nExpected:\n");
-	  print_hex(hash->digest_size, digest->data);
+	  print_hex(digest->length, digest->data);
 	  abort();
 	}      
     }
@@ -559,6 +953,69 @@ test_hash_large(const struct nettle_hash *hash,
   free(data);
 }
 
+void
+test_mac(const struct nettle_mac *mac,
+	 const struct tstring *key,
+	 const struct tstring *msg,
+	 const struct tstring *digest)
+{
+  void *ctx = xalloc(mac->context_size);
+  uint8_t *hash = xalloc(mac->digest_size);
+  unsigned i;
+
+  ASSERT (digest->length <= mac->digest_size);
+  ASSERT (key->length == mac->key_size);
+  mac->set_key (ctx, key->data);
+  mac->update (ctx, msg->length, msg->data);
+  mac->digest (ctx, digest->length, hash);
+
+  if (!MEMEQ (digest->length, digest->data, hash))
+    {
+      fprintf (stderr, "test_mac failed, msg: ");
+      print_hex (msg->length, msg->data);
+      fprintf(stderr, "Output:");
+      print_hex (mac->digest_size, hash);
+      fprintf(stderr, "Expected:");
+      tstring_print_hex(digest);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  /* attempt to re-use the structure */
+  mac->update (ctx, msg->length, msg->data);
+  mac->digest (ctx, digest->length, hash);
+  if (!MEMEQ (digest->length, digest->data, hash))
+    {
+      fprintf (stderr, "test_mac: failed on re-use, msg: ");
+      print_hex (msg->length, msg->data);
+      fprintf(stderr, "Output:");
+      print_hex (mac->digest_size, hash);
+      fprintf(stderr, "Expected:");
+      tstring_print_hex(digest);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+
+  /* attempt byte-by-byte hashing */
+  mac->set_key (ctx, key->data);
+  for (i=0;i<msg->length;i++)
+    mac->update (ctx, 1, msg->data+i);
+  mac->digest (ctx, digest->length, hash);
+  if (!MEMEQ (digest->length, digest->data, hash))
+    {
+      fprintf (stderr, "cmac_hash failed on byte-by-byte, msg: ");
+      print_hex (msg->length, msg->data);
+      fprintf(stderr, "Output:");
+      print_hex (16, hash);
+      fprintf(stderr, "Expected:");
+      tstring_print_hex(digest);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
+  free (ctx);
+  free (hash);
+}
+
 void
 test_armor(const struct nettle_armor *armor,
            size_t data_length,
@@ -566,7 +1023,7 @@ test_armor(const struct nettle_armor *armor,
            const char *ascii)
 {
   size_t ascii_length = strlen(ascii);
-  uint8_t *buffer = xalloc(1 + ascii_length);
+  char *buffer = xalloc(1 + ascii_length);
   uint8_t *check = xalloc(1 + armor->decode_length(ascii_length));
   void *encode = xalloc(armor->encode_context_size);
   void *decode = xalloc(armor->decode_context_size);
@@ -606,19 +1063,6 @@ test_armor(const struct nettle_armor *armor,
 
 #if WITH_HOGWEED
 
-#ifndef mpn_zero_p
-int
-mpn_zero_p (mp_srcptr ap, mp_size_t n)
-{
-  while (--n >= 0)
-    {
-      if (ap[n] != 0)
-	return 0;
-    }
-  return 1;
-}
-#endif
-
 void
 mpn_out_str (FILE *f, int base, const mp_limb_t *xp, mp_size_t xn)
 {
@@ -639,7 +1083,7 @@ mpz_urandomb (mpz_t r, struct knuth_lfib_ctx *ctx, mp_bitcnt_t bits)
   uint8_t *buf = xalloc (bytes);
 
   knuth_lfib_random (ctx, bytes, buf);
-  buf[bytes-1] &= 0xff >> (8*bytes - bits);
+  buf[0] &= 0xff >> (8*bytes - bits);
   nettle_mpz_set_str_256_u (r, bytes, buf);
   free (buf);
 }
@@ -1212,12 +1656,15 @@ test_dsa_key(const struct dsa_params *params,
 }
 
 const struct ecc_curve * const ecc_curves[] = {
-  &nettle_secp_192r1,
-  &nettle_secp_224r1,
-  &nettle_secp_256r1,
-  &nettle_secp_384r1,
-  &nettle_secp_521r1,
+  &_nettle_secp_192r1,
+  &_nettle_secp_224r1,
+  &_nettle_secp_256r1,
+  &_nettle_secp_384r1,
+  &_nettle_secp_521r1,
   &_nettle_curve25519,
+  &_nettle_curve448,
+  &_nettle_gost_gc256b,
+  &_nettle_gost_gc512a,
   NULL
 };
 
@@ -1265,80 +1712,140 @@ test_ecc_point (const struct ecc_curve *ecc,
     }
 }
 
-void
-test_ecc_mul_a (unsigned curve, unsigned n, const mp_limb_t *p)
-{
-  /* For each curve, the points 2 g, 3 g and 4 g */
-  static const struct ecc_ref_point ref[6][3] = {
-    { { "dafebf5828783f2ad35534631588a3f629a70fb16982a888",
+/* For each curve, the points g, 2 g, 3 g and 4 g */
+static const struct ecc_ref_point ecc_ref[9][4] = {
+  { { "188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012",
+      "07192b95ffc8da78631011ed6b24cdd573f977a11e794811" },
+    { "dafebf5828783f2ad35534631588a3f629a70fb16982a888",
 	"dd6bda0d993da0fa46b27bbc141b868f59331afa5c7e93ab" },
-      { "76e32a2557599e6edcd283201fb2b9aadfd0d359cbb263da",
+    { "76e32a2557599e6edcd283201fb2b9aadfd0d359cbb263da",
 	"782c37e372ba4520aa62e0fed121d49ef3b543660cfd05fd" },
-      { "35433907297cc378b0015703374729d7a4fe46647084e4ba",
+    { "35433907297cc378b0015703374729d7a4fe46647084e4ba",
 	"a2649984f2135c301ea3acb0776cd4f125389b311db3be32" }
-    },
-    { { "706a46dc76dcb76798e60e6d89474788d16dc18032d268fd1a704fa6",
+  },
+  { { "b70e0cbd6bb4bf7f321390b94a03c1d356c21122343280d6115c1d21",
+	"bd376388b5f723fb4c22dfe6cd4375a05a07476444d5819985007e34" },
+    { "706a46dc76dcb76798e60e6d89474788d16dc18032d268fd1a704fa6",
 	"1c2b76a7bc25e7702a704fa986892849fca629487acf3709d2e4e8bb" },
-      { "df1b1d66a551d0d31eff822558b9d2cc75c2180279fe0d08fd896d04",
+    { "df1b1d66a551d0d31eff822558b9d2cc75c2180279fe0d08fd896d04",
 	"a3f7f03cadd0be444c0aa56830130ddf77d317344e1af3591981a925" },
-      { "ae99feebb5d26945b54892092a8aee02912930fa41cd114e40447301",
+    { "ae99feebb5d26945b54892092a8aee02912930fa41cd114e40447301",
 	"482580a0ec5bc47e88bc8c378632cd196cb3fa058a7114eb03054c9" },
-    },
-    { { "7cf27b188d034f7e8a52380304b51ac3c08969e277f21b35a60b48fc47669978",
+  },
+  { { "6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296",
+	"4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5" },
+    { "7cf27b188d034f7e8a52380304b51ac3c08969e277f21b35a60b48fc47669978",
 	"7775510db8ed040293d9ac69f7430dbba7dade63ce982299e04b79d227873d1" },
-      { "5ecbe4d1a6330a44c8f7ef951d4bf165e6c6b721efada985fb41661bc6e7fd6c",
+    { "5ecbe4d1a6330a44c8f7ef951d4bf165e6c6b721efada985fb41661bc6e7fd6c",
 	"8734640c4998ff7e374b06ce1a64a2ecd82ab036384fb83d9a79b127a27d5032" },
-      { "e2534a3532d08fbba02dde659ee62bd0031fe2db785596ef509302446b030852",
+    { "e2534a3532d08fbba02dde659ee62bd0031fe2db785596ef509302446b030852",
 	"e0f1575a4c633cc719dfee5fda862d764efc96c3f30ee0055c42c23f184ed8c6" },
-    },
-    { { "8d999057ba3d2d969260045c55b97f089025959a6f434d651d207d19fb96e9e"
+  },
+  { { "aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a38"
+	"5502f25dbf55296c3a545e3872760ab7",
+	"3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c0"
+	"0a60b1ce1d7e819d7a431d7c90ea0e5f" },
+    { "8d999057ba3d2d969260045c55b97f089025959a6f434d651d207d19fb96e9e"
 	"4fe0e86ebe0e64f85b96a9c75295df61",
 	"8e80f1fa5b1b3cedb7bfe8dffd6dba74b275d875bc6cc43e904e505f256ab425"
 	"5ffd43e94d39e22d61501e700a940e80" },
-      { "77a41d4606ffa1464793c7e5fdc7d98cb9d3910202dcd06bea4f240d3566da6"
+    { "77a41d4606ffa1464793c7e5fdc7d98cb9d3910202dcd06bea4f240d3566da6"
 	"b408bbae5026580d02d7e5c70500c831",
 	"c995f7ca0b0c42837d0bbe9602a9fc998520b41c85115aa5f7684c0edc111eac"
 	"c24abd6be4b5d298b65f28600a2f1df1" },
-      { "138251cd52ac9298c1c8aad977321deb97e709bd0b4ca0aca55dc8ad51dcfc9d"
+    { "138251cd52ac9298c1c8aad977321deb97e709bd0b4ca0aca55dc8ad51dcfc9d"
 	"1589a1597e3a5120e1efd631c63e1835",
 	"cacae29869a62e1631e8a28181ab56616dc45d918abc09f3ab0e63cf792aa4dc"
 	"ed7387be37bba569549f1c02b270ed67" },
-    },
-    { { "43"
+  },
+  { { "c6"
+	"858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d3dba"
+	"a14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31c2e5bd66",
+	"118"
+	"39296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e662c"
+	"97ee72995ef42640c550b9013fad0761353c7086a272c24088be94769fd16650" },
+    { "43"
 	"3c219024277e7e682fcb288148c282747403279b1ccc06352c6e5505d769be97"
 	"b3b204da6ef55507aa104a3a35c5af41cf2fa364d60fd967f43e3933ba6d783d",
 	"f4"
 	"bb8cc7f86db26700a7f3eceeeed3f0b5c6b5107c4da97740ab21a29906c42dbb"
 	"b3e377de9f251f6b93937fa99a3248f4eafcbe95edc0f4f71be356d661f41b02"
-      },
-      { "1a7"
+    },
+    { "1a7"
 	"3d352443de29195dd91d6a64b5959479b52a6e5b123d9ab9e5ad7a112d7a8dd1"
 	"ad3f164a3a4832051da6bd16b59fe21baeb490862c32ea05a5919d2ede37ad7d",
 	"13e"
 	"9b03b97dfa62ddd9979f86c6cab814f2f1557fa82a9d0317d2f8ab1fa355ceec"
 	"2e2dd4cf8dc575b02d5aced1dec3c70cf105c9bc93a590425f588ca1ee86c0e5" },
-      { "35"
+    { "35"
 	"b5df64ae2ac204c354b483487c9070cdc61c891c5ff39afc06c5d55541d3ceac"
 	"8659e24afe3d0750e8b88e9f078af066a1d5025b08e5a5e2fbc87412871902f3",
 	"82"
 	"096f84261279d2b673e0178eb0b4abb65521aef6e6e32e1b5ae63fe2f19907f2"
 	"79f283e54ba385405224f750a95b85eebb7faef04699d1d9e21f47fc346e4d0d" },
-    },
-    { { "36ab384c9f5a046c3d043b7d1833e7ac080d8e4515d7a45f83c5a14e2843ce0e",
+  },
+  { { "216936d3cd6e53fec0a4e231fdd6dc5c692cc7609525a7b2c9562d608f25d51a",
+      "6666666666666666666666666666666666666666666666666666666666666658" },
+    { "36ab384c9f5a046c3d043b7d1833e7ac080d8e4515d7a45f83c5a14e2843ce0e",
 	"2260cdf3092329c21da25ee8c9a21f5697390f51643851560e5f46ae6af8a3c9" },
-      { "67ae9c4a22928f491ff4ae743edac83a6343981981624886ac62485fd3f8e25c",
+    { "67ae9c4a22928f491ff4ae743edac83a6343981981624886ac62485fd3f8e25c",
 	"1267b1d177ee69aba126a18e60269ef79f16ec176724030402c3684878f5b4d4" },
-      { "203da8db56cff1468325d4b87a3520f91a739ec193ce1547493aa657c4c9f870",
+    { "203da8db56cff1468325d4b87a3520f91a739ec193ce1547493aa657c4c9f870",
 	"47d0e827cb1595e1470eb88580d5716c4cf22832ea2f0ff0df38ab61ca32112f" },
-    }
-  };
-  assert (curve < 6);
+  },
+  { { "4f1970c66bed0ded221d15a622bf36da9e146570470f1767ea6de324a3d3a46412ae1af72ab66511433b80e18b00938e2626a82bc70cc05e",
+	"693f46716eb6bc248876203756c9c7624bea73736ca3984087789c1e05a0c2d73ad3ff1ce67c39c4fdbd132c4ed7c8ad9808795bf230fa14" },
+    { "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa955555555555555555555555555555555555555555555555555555555",
+	"ae05e9634ad7048db359d6205086c2b0036ed7a035884dd7b7e36d728ad8c4b80d6565833a2a3098bbbcb2bed1cda06bdaeafbcdea9386ed" },
+    { "865886b9108af6455bd64316cb6943332241b8b8cda82c7e2ba077a4a3fcfe8daa9cbf7f6271fd6e862b769465da8575728173286ff2f8f",
+	"e005a8dbd5125cf706cbda7ad43aa6449a4a8d952356c3b9fce43c82ec4e1d58bb3a331bdb6767f0bffa9a68fed02dafb822ac13588ed6fc" },
+    { "49dcbc5c6c0cce2c1419a17226f929ea255a09cf4e0891c693fda4be70c74cc301b7bdf1515dd8ba21aee1798949e120e2ce42ac48ba7f30",
+	"d49077e4accde527164b33a5de021b979cb7c02f0457d845c90dc3227b8a5bc1c0d8f97ea1ca9472b5d444285d0d4f5b32e236f86de51839" },
+  },
+  { { "0000000000000000000000000000000000000000000000000000000000000001",
+      "8d91e471e0989cda27df505a453f2b7635294f2ddf23e3b122acc99c9e9f1e14" },
+    { "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffd95",
+      "726e1b8e1f676325d820afa5bac0d489cad6b0d220dc1c4edd5336636160df83" },
+    { "8e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38e38d2c",
+      "76bcd1ca9a23b041d4d9baf507a6cd821267a94c838768e8486117796b788a51" },
+    { "f7063e7063e7063e7063e7063e7063e7063e7063e7063e7063e7063e7063e4b7",
+      "83ccf17ba6706d73625cc3534c7a2b9d6ec1ee6a9a7e07c10d84b388de59f741" },
+  },
+  { { "0000000000000000000000000000000000000000000000000000000000000000"
+      "0000000000000000000000000000000000000000000000000000000000000003",
+      "7503cfe87a836ae3a61b8816e25450e6ce5e1c93acf1abc1778064fdcbefa921"
+      "df1626be4fd036e93d75e6a50e3a41e98028fe5fc235f5b889a589cb5215f2a4" },
+    { "3b89dcfc622996ab97a5869dbff15cf51db00954f43a58a5e5f6b0470a132b2f"
+      "4434bbcd405d2a9516151d2a6a04f2e4375bf48de1fdb21fb982afd9d2ea137c",
+      "c813c4e2e2e0a8a391774c7903da7a6f14686e98e183e670ee6fb784809a3e92"
+      "ca209dc631d85b1c7534ed3b37fddf64d854d7e01f91f18bb3fd307591afc051" },
+    { "a1ff1ab2712a267eb53935ddb5a567f84db156cc096168a1174291d5f488fba5"
+      "43d2840b4d2dd35d764b2f57b308907aec55cfba10544e8416e134687ccb87c3",
+      "3cb5c4417ec4637f30374f189bb5b984c41e3a48d7f84fbfa3819e3f333f7eb3"
+      "11d3af7e67c4c16eeacfac2fe94c6dd4c6366f711a4fb6c7125cd7ec518d90d6" },
+    { "b7bfb80956c8670031ba191929f64e301d681634236d47a60e571a4bedc0ef25"
+      "7452ef78b5b98dbb3d9f3129d9349433ce2a3a35cb519c91e2d633d7b373ae16",
+      "3bee95e29eecc5d5ad2beba941abcbf9f1cad478df0fecf614f63aeebef77850"
+      "da7efdb93de8f3df80bc25eac09239c14175f5c29704ce9a3e383f1b3ec0e929" },
+  }
+};
+
+void
+test_ecc_ga (unsigned curve, const mp_limb_t *p)
+{
+  return test_ecc_point (ecc_curves[curve], &ecc_ref[curve][0], p);
+}
+
+void
+test_ecc_mul_a (unsigned curve, unsigned n, const mp_limb_t *p)
+{
+  assert (curve < 9);
   assert (n <= 4);
   if (n == 0)
     {
       /* Makes sense for curve25519 only */
       const struct ecc_curve *ecc = ecc_curves[curve];
-      assert (ecc->p.bit_size == 255);
+      assert (ecc->p.bit_size == 255 || ecc->p.bit_size == 448);
       if (!mpn_zero_p (p, ecc->p.size)
 	  || mpn_cmp (p + ecc->p.size, ecc->unit, ecc->p.size) != 0)
 	{
@@ -1352,29 +1859,8 @@ test_ecc_mul_a (unsigned curve, unsigned n, const mp_limb_t *p)
 	  abort();
 	}
     }
-  else if (n == 1)
-    {
-      const struct ecc_curve *ecc = ecc_curves[curve];
-      if (mpn_cmp (p, ecc->g, 2*ecc->p.size) != 0)
-	{
-	  fprintf (stderr, "Incorrect point (expected g)!\n"
-		   "got: x = ");
-	  write_mpn (stderr, 16, p, ecc->p.size);
-	  fprintf (stderr, "\n"
-		   "     y = ");
-	  write_mpn (stderr, 16, p + ecc->p.size, ecc->p.size);
-	  fprintf (stderr, "\n"
-		   "ref: x = ");
-	  write_mpn (stderr, 16, ecc->g, ecc->p.size);
-	  fprintf (stderr, "\n"
-		   "     y = ");
-	  write_mpn (stderr, 16, ecc->g + ecc->p.size, ecc->p.size);
-	  fprintf (stderr, "\n");
-	  abort();
-	}
-    }
   else
-    test_ecc_point (ecc_curves[curve], &ref[curve][n-2], p);
+    test_ecc_point (ecc_curves[curve], &ecc_ref[curve][n-1], p);
 }
 
 void
@@ -1391,5 +1877,46 @@ test_ecc_mul_h (unsigned curve, unsigned n, const mp_limb_t *p)
   free (scratch);
 }
 
+void
+test_ecc_get_g (unsigned curve, mp_limb_t *rp)
+{
+  const struct ecc_curve *ecc = ecc_curves[curve];
+  mpz_t x;
+  mpz_t y;
+  mpz_init_set_str (x, ecc_ref[curve][0].x, 16);
+  mpz_init_set_str (y, ecc_ref[curve][0].y, 16);
+
+  if (ecc->use_redc)
+    {
+      mpz_t t;
+      mpz_mul_2exp (x, x, ecc->p.size * GMP_NUMB_BITS);
+      mpz_mod (x, x, mpz_roinit_n (t, ecc->p.m, ecc->p.size));
+      mpz_mul_2exp (y, y, ecc->p.size * GMP_NUMB_BITS);
+      mpz_mod (y, y, mpz_roinit_n (t, ecc->p.m, ecc->p.size));
+    }
+  mpz_limbs_copy (rp, x, ecc->p.size);
+  mpz_limbs_copy (rp + ecc->p.size, y, ecc->p.size);
+  mpn_copyi (rp + 2*ecc->p.size, ecc->unit, ecc->p.size);
+
+  mpz_clear (x);
+  mpz_clear (y);
+}
+
+void
+test_ecc_get_ga (unsigned curve, mp_limb_t *rp)
+{
+  const struct ecc_curve *ecc = ecc_curves[curve];
+  mpz_t x;
+  mpz_t y;
+  mpz_init_set_str (x, ecc_ref[curve][0].x, 16);
+  mpz_init_set_str (y, ecc_ref[curve][0].y, 16);
+
+  mpz_limbs_copy (rp, x, ecc->p.size);
+  mpz_limbs_copy (rp + ecc->p.size, y, ecc->p.size);
+
+  mpz_clear (x);
+  mpz_clear (y);
+}
+
 #endif /* WITH_HOGWEED */
 
